@@ -70,6 +70,7 @@ static int long_docs_mode = FALSE;
 static int schema_name_mode = FALSE;
 static int associate_schema_mode = FALSE;
 static int dissociate_schema_mode = FALSE;
+static int ignore_schema_defaults = FALSE;
 static int default_source_mode = FALSE;
 static int recursive_unset_mode = FALSE;
 static int do_version = FALSE;
@@ -409,6 +410,15 @@ struct poptOption options[] = {
     N_("Remove any schema name applied to the given keys"),
     NULL
   },
+  {
+    "ignore-schema-defaults",
+    '\0',
+    POPT_ARG_NONE,
+    &ignore_schema_defaults,
+    0,
+    N_("Ignore schema defaults when reading values."),
+    NULL
+  },
   {
     "get-default-source",
     '\0',
@@ -588,7 +598,7 @@ main (int argc, char** argv)
     {
       g_printerr (_("--set_schema should not be used with --get, --set, --unset, --all-entries, --all-dirs\n"));
       return 1;
-    }  
+    }
 
   if ((value_type != NULL) && !(set_mode || set_schema_mode))
     {
@@ -602,6 +612,14 @@ main (int argc, char** argv)
       return 1;
     }
 
+  if (ignore_schema_defaults && !(get_mode || all_entries_mode ||
+				  dump_values || recursive_list ||
+				  get_list_size_mode || get_list_element_mode))
+    {
+      g_printerr (_("--ignore-schema-defaults is only relevant with --get, --all-entries, --dump, --recursive-list, --get-list-size or --get-list-element\n"));
+      return 1;
+    }
+
   if (ping_gconfd && (shutdown_gconfd || set_mode || get_mode || unset_mode ||
                       all_subdirs_mode || all_entries_mode || recursive_list || 
                       get_type_mode || get_list_size_mode || get_list_element_mode ||
@@ -703,12 +721,6 @@ main (int argc, char** argv)
       return 1;
     }
 
-  if (config_source && !use_local_source)
-    {
-      g_printerr (_("You should use --direct when using a non-default configuration source\n"));
-      return 1;
-    }
-  
   if (!gconf_init(argc, argv, &err))
     {
       g_printerr (_("Failed to init GConf: %s\n"), err->message);
@@ -780,10 +792,16 @@ main (int argc, char** argv)
     conf = gconf_engine_get_default();
   else
     {
+      GSList *addresses;
+
+      addresses = gconf_persistent_name_get_address_list (config_source);
+
       if (use_local_source)
-        conf = gconf_engine_get_local(config_source, &err);
+        conf = gconf_engine_get_local_for_addresses (addresses, &err);
       else
-        conf = gconf_engine_get_for_address(config_source, &err);
+        conf = gconf_engine_get_for_addresses (addresses, &err);
+
+      gconf_address_list_free (addresses);
     }
   
   if (conf == NULL)
@@ -1226,7 +1244,8 @@ list_pairs_in_dir(GConfEngine* conf, const gchar* dir, guint depth)
           GConfEntry* pair = tmp->data;
           gchar* s;
 
-          if (gconf_entry_get_value (pair))
+          if (gconf_entry_get_value (pair) && 
+	      (!ignore_schema_defaults || !gconf_entry_get_is_default (pair)))
             s = gconf_value_to_string (gconf_entry_get_value (pair));
           else
             s = g_strdup(_("(no value set)"));
@@ -1497,7 +1516,8 @@ dump_entries_in_dir(GConfEngine* conf, const gchar* dir, const gchar* base_dir)
         g_print ("      <schema_key>%s</schema_key>\n",
 		 get_key_relative(gconf_entry_get_schema_name(entry), base_dir));
 
-      if (entry->value)
+      if (entry->value && 
+	  (!ignore_schema_defaults || !gconf_entry_get_is_default(entry)))
         print_value_in_xml(entry->value, 6);
 
       g_print ("    </entry>\n");
@@ -1541,6 +1561,21 @@ do_spawn_daemon(GConfEngine* conf)
     }
 }
 
+static inline GConfValue *
+get_maybe_without_default (GConfEngine  *conf,
+			   const gchar  *key,
+			   GError      **err)
+{
+  if (!ignore_schema_defaults)
+    {
+      return gconf_engine_get (conf, key, err);
+    }
+  else
+    {
+      return gconf_engine_get_without_default (conf, key, err);
+    }
+}
+
 static int
 do_get(GConfEngine* conf, const gchar** args)
 {
@@ -1559,7 +1594,7 @@ do_get(GConfEngine* conf, const gchar** args)
 
       err = NULL;
 
-      value = gconf_engine_get (conf, *args, &err);
+      value = get_maybe_without_default (conf, *args, &err);
          
       if (value != NULL)
         {
@@ -1818,7 +1853,7 @@ do_get_type(GConfEngine* conf, const gchar** args)
 
       err = NULL;
 
-      value = gconf_engine_get (conf, *args, &err);
+      value = get_maybe_without_default (conf, *args, &err);
          
       if (value != NULL)
 	{
@@ -1856,7 +1891,7 @@ do_get_list_size(GConfEngine* conf, const gchar** args)
       return 1;
     }
 
-  list = gconf_engine_get (conf, *args, &err);
+  list = get_maybe_without_default (conf, *args, &err);
 
   if (list == NULL)
     {
@@ -1901,7 +1936,7 @@ do_get_list_element(GConfEngine* conf, const gchar** args)
       return 1;
     }
 
-  list = gconf_engine_get (conf, *args, &err);
+  list = get_maybe_without_default (conf, *args, &err);
 
   if (list == NULL)
     {
